(function($){
    $.fn.ddup = function(prms) {
        prms = $.extend({
            //parameters:
            maxfiles: 20, //maximum available number of files
            allow_exts: ['jpg', 'jpeg', 'png', 'gif'], //allowed image file formats
            maxfilesize: 10 * 1024 * 1024, //maximum available file size in bytes
            preview_w: 100, //width of the preview image in pixels
            preview_h: 100, //height of the preview image in pixels
            files_arrname: 'ddup_files[]', //name of the $_FILES subarray sending with xhr request
            php_path: '/upload.php', //path to the php handler
            files_varname: 'ddup_files', //input name of $_FILES array
            submit_delay: 0, //delay between finish of xhr files uploading and submitting form
            lang: 'en', //interface language
            lang_overwrites: {}, //language overwrites

            //triggers:
            on_before_submit: function() {}
        }, prms);
        prms.preview_w = parseInt(prms.preview_w);
        prms.preview_h = parseInt(prms.preview_h);
        
        var langs = {
            en: {
                drag: 'Drag files here or click',
                drag_1: 'Drag file here or click',
                drop: 'Drop files here',
                drop_1: 'Drop file here',
                mb: 'MB',
                remove: 'remove',
                err_maxfiles_1: '* file maximum',
                err_maxfiles: '* files maximum',
                err_maxfilesize: "File '*' exceed the maximum allowable size in ** MB",
                err_read: "Can not read file '*'",
                err_ext: "File '*' has an incorrect format. Use ** files",
                err_added: "File '*' has already been added",
                err_send_files: 'An error has occurred while sending the files',
            },
            de: {
                drag: 'Ziehen Sie die Dateien hier oder Klicken',
                drag_1: 'Ziehen Sie eine Datei hier oder Klicken',
                drop: 'Dateien hier ablegen',
                drop_1: 'Datei hier ablegen',
                mb: 'MB',
                remove: 'löschen',
                err_maxfiles_1: '* Datei maximal',
                err_maxfiles: '* Dateien maximal',
                err_maxfilesize: "Datei '*' überschreiten die maximal zulässige Größe pro ** MB",
                err_read: "Die Datei kann nicht 1 zu lesen",
                err_ext: "Datei '*' hat ein falsches Format. Verwenden ** Dateien",
                err_added: "Datei '*' wurde bereits hinzugefügt",
                err_send_files: 'Ein Fehler ist aufgetreten, während das Senden von Dateien',
            },
            ru: {
                drag: 'Перетащите файлы сюда или кликните',
                drag_1: 'Перетащите файл сюда или кликните',
                drop: 'Сбросьте файлы сюда',
                drop_1: 'Сбросьте файл сюда',
                mb: 'MB',
                remove: 'удалить',
                err_maxfiles_1: 'Максимум * файл',
                err_maxfiles: 'Максимум * файла(ов)',
                err_maxfilesize: "Файл '*' превышает максимально допустимый размер в ** мегабайт",
                err_read: "Файл '*' не может быть прочитан",
                err_ext: "Файл '*' имеет неверный формат. Используйте ** файлы",
                err_added: "Файл '*' уже был добавлен",
                err_send_files: 'При отправке файлов произошла ошибка',
            }
        };
        var lang = ( langs[prms.lang] === undefined ) ? langs.en : langs[prms.lang];
        for(var key in prms.lang_overwrites) {
            if( lang[key] !== undefined ) lang[key] = prms.lang_overwrites[key];
        }
        if(prms.maxfiles == 1) {
            lang.drag = lang.drag_1;
            lang.drop = lang.drop_1;
        }
        lang.err_maxfiles = ((prms.maxfiles > 1) ? lang.err_maxfiles : lang.err_maxfiles_1).replace('*', prms.maxfiles);
        lang.err_maxfilesize = lang.err_maxfilesize.replace('**', parseInt(prms.maxfilesize / 1024 / 1024));
        lang.err_ext = lang.err_ext.replace('**', prms.allow_exts.join(', ').toUpperCase());

        var obj = this.first();
        var form = obj.closest('form');
        obj.html(
            '<div class="ddup-base">' +
                '<ul></ul>' +
                '<div class="ddup-guide">' +
                    '<div>' + lang.drag + '</div>' +
                    '<span hidden>×</span>' + //not pseudo coz Safari does not support cursor's styles for pseudo elements
                '</div>' +
                '<div class="ddup-catcher"></div>' +
            '</div>'
        );
        obj.addClass('noneselect');
        var multiple = (
            prms.maxfiles == 1 ||
            navigator.userAgent.search("Safari") >= 0 && navigator.userAgent.search("Chrome") < 0 //coz Safari can't detect size of files in multiple selecting
        ) ? '' : 'multiple';
        obj.after('<input type="file" accept="image/*" style="height: 0; width: 0; margin: -2px" ' + multiple + '>'); //not hidden coz of Safari
        obj.cansel = false;
        var base = obj.find('.ddup-base');
        var prevlist = base.find('ul');
        var guide = base.find('.ddup-guide');
        var guide_box = guide.children('div');
        var catcher = base.find('.ddup-catcher');
        var input = obj.next('input');
        var files_to_send = new Array();
        var errors = new Array();
        var passed = 0;
        var file_reader = (window.FileReader === undefined) ? false : true;
        var files_sent = false;
        
        obj.on({
            'dragover drop': function() { return false; },
            click: function() { if(!obj.hasClass('ddup-inactive') && !guide.hasClass('ddup-errors')) input.click(); },
            dragenter: function() {
                if(!obj.hasClass('ddup-inactive')) catcher.css('pointer-events', 'auto');
                if(guide.hasClass('ddup-errors')) guide.click();
            }
        });
        
        input.on('change', function() { handle_adding(this.files); });
        
        var blinkflag = true;
        catcher.on({
            dragover: function() {
                if(blinkflag) {
                    blinkflag = false;
                    guide.fadeOut(500, function() {
                        base.addClass('ddup-dragover');
                        guide_box.html(lang.drop);
                        guide.fadeIn(500);
                        blinkflag = true;
                    });
                }
            },
            dragleave: function() {
                catcher.css('pointer-events', 'none');
                guide.fadeOut(500, function() {
                    base.removeClass('ddup-dragover');
                    guide.clear();
                });
            },
            drop: function(e) {
                var files = e.originalEvent.dataTransfer.files;
                if(files.length === 0) {
                    $(this).trigger('dragleave');
                    return false;
                }
                blinkflag = true;
                catcher.css('pointer-events', 'none');
                base.removeClass('ddup-dragover');
                handle_adding(files);
            }
        });
        
        prevlist.on('click', 'li', function(e) {
            if(obj.hasClass('ddup-inactive')) return this;
            if(!$(e.target).hasClass('ddup-del_prev')) return false;
            
            var previw = $(this).closest('li');
            files_to_send.splice(previw.index(), 1);
            previw.fadeOut(500, function() {
                previw.remove();
                guide.clear();
            });
            return false;
        });
        
        guide.on('click', function() {
            if(!guide.hasClass('ddup-errors')) return this;
            
            base.removeClass('ddup-covered');
            guide.fadeOut(500, function() {
                guide.removeClass('ddup-errors');
                guide.clear();
            });
        });
        guide.clear = function() {
            guide_box.html('');
            if(files_to_send.length === 0) {
                guide_box.html(lang.drag);
                guide.hide().fadeIn(500);
            }
        };
        
        function handle_adding(files) {
            obj.addClass('ddup-inactive');
            guide.stop(true).fadeOut(500, function() {
                input.replaceWith( input = input.clone(true) ); //fix for GH and Safari
                handle_files(files);
            });
        }
        
        function handle_files(files) {
            var fts_sum = files_to_send.length;
            passed = 0;
            
            for(var i = 0, file; file = files[i]; i++) {
                //check number of already added files:
                if((fts_sum + passed) >= prms.maxfiles) {
                    errors.push(lang.err_maxfiles);
                    break;
                }
                
                //check file size:
                if(file.size > prms.maxfilesize) {
                    errors.push(lang.err_maxfilesize.replace('*', file.name));
                    continue;
                }
                
                //define and check file extension:
                file.ext = file.type.replace('image/', '');
                if($.inArray(file.ext, prms.allow_exts) === -1) {
                    errors.push(lang.err_ext.replace('*', file.name));
                    continue;
                }
               
                passed++;
                read_file(file);
            }
            
            if(passed === 0) show_errors();
        }
        
        function read_file(file) {
            if(file_reader) {
                var reader = new FileReader();
                reader.onloadstart = function() {
                    //add preloader:
                    var preloader_class = 'ddup-preloader-' + prevlist.children().last().index();
                    prevlist.append('<li class="' + preloader_class + '" style="width: ' + prms.preview_w + 'px; height: ' + prms.preview_h + 'px; line-height: ' + prms.preview_h + 'px">0%</li>');
                    file.preloader = '.' + preloader_class;
                }
                reader.onprogress = function(e) {
                    if(e.lengthComputable) prevlist.children(file.preloader).html(parseInt(100 * e.loaded / e.total) + '%');
                };
                reader.onload = (function(file) {
                    return function(e) {
                        prevlist.children(file.preloader).html('100%');
                        file.src = e.target.result;
                        //check whether file is already added:
                        var unique = true;
                        files_to_send.forEach(function(item) {
                            if(item.img.src == file.src) {
                                unique = false;
                                abort_file_handling(file, lang.err_added.replace('*', item.name));
                            }
                        });
                        if(unique) add_file_img(file);
                    };
                })(file);
                reader.readAsDataURL(file);
            }
            else {
                //check (without FileReader data) whether file is already added:
                if(is_obj_unique(file, files_to_send))
                    show_preview(file);
                else
                    abort_file_handling(file, lang.err_added.replace('*', file.name));
            }
        }
        
        function is_obj_unique(obj, set) {
            for(var i = 0, o; o = set[i]; i++) {
                var props = 0;
                var sims = 0;
                for(var prop in o) {
                    props++;
                    if(String(obj[prop]) === String(o[prop])) sims++;
                    else break;
                }
                if(props === sims) return false;
            }
            return true;
        }
        
        function add_file_img(file) {
            var img = new Image();
            img.src = file.src;
            img.onerror = function() {
                abort_file_handling(file, lang.err_read.replace('*', file.name));
            }
            img.onload = function() {
                file.img = img;
                delete file.src; //coz now src is in file.img.src
                show_preview(file);
            }
        }
        
        function abort_file_handling(file, error) {
            prevlist.children(file.preloader).fadeOut(500, function() { $(this).remove(); });
            errors.push(error);
            passed--;
            if(passed === 0) show_errors();
        }
       
        function show_preview(file) {
            var li = $(
                '<li>' +
                    '<div>' +
                        '<canvas width="' + prms.preview_w + '" height="' + prms.preview_h + '"></canvas>' +
                        '<div>' + file.name + '</div>' +
                    '</div>' +
                    '<span>' + (file.size / 1024 / 1024).toFixed(2) + ' ' + lang.mb + '</span>' +
                    '<span class="ddup-del_prev">' + lang.remove + '</span>' +
                '</li>'
            );
            prevlist.children(file.preloader).replaceWith(li);
            if(file.img === undefined) {
                li.find('canvas + div').fadeTo(0, 1);
            }
            else {
                fill_canvas(li.find('canvas').get(0), file.img);
            }
            li.fadeTo(500, 1, function() { posthandle_file(file); });
        }
        
        function fill_canvas(canvas, img) {
            var h_orig = img.height;
            var w_orig = img.width;
            var h_prev = prms.preview_h;
            var w_prev = prms.preview_w;
            if((h_prev / h_orig) <= (w_prev / w_orig)) {
                var w_frag = w_orig;
                var h_frag = parseInt((w_orig * h_prev) / w_prev);
                var w_frag_start = 0;
                var h_frag_start = parseInt((h_orig - h_frag) / 2);
            }
            else {            
                var h_frag = h_orig;
                var w_frag = parseInt((h_orig * w_prev) / h_prev);
                var h_frag_start = 0;
                var w_frag_start = parseInt((w_orig - w_frag) / 2);
            }
            canvas.getContext('2d').drawImage(img, w_frag_start, h_frag_start, w_frag, h_frag, 0, 0, w_prev, h_prev);
        }
        
        function posthandle_file(file) {
            files_to_send.push(file);
            if(files_to_send.length < passed) return;
            
            show_errors();
        }
        
        function show_errors() {
            if(errors.length === 0) {
                obj.removeClass('ddup-inactive');
                return;
            }
            
            var report = '';
            errors.forEach(function(el) { report += el + '<br>'; });
            errors = new Array();
            setTimeout(function() {
                base.addClass('ddup-covered');
                guide_box.html(report);
                guide.addClass('ddup-errors').fadeIn(500, function() { obj.removeClass('ddup-inactive'); });
            }, 500);
        }
        
        form.submit(function() {
            prms.on_before_submit(obj);
            if(obj.cansel) return true;
            if(obj.hasClass('ddup-inactive') && !files_sent) return false;
            if(guide.hasClass('ddup-errors')) {
                guide.fadeOut(500, function() {
                    guide.removeClass('ddup-errors');
                    form.submit();
                });
                return false;
            }
            if(files_sent || files_to_send.length == 0) return true;

            obj.addClass('ddup-inactive');
            var xhr = new XMLHttpRequest();
            xhr.open('POST', prms.php_path, true);
            var fd = new FormData;
            for(var i = 0, file; file = files_to_send[i]; i++) {
                fd.append(prms.files_arrname, file);
            }
            
            base.addClass('ddup-covered');
            guide.addClass('ddup-progressbar');
            guide_box.html('<div>0%</div><div></div>');
            var div_perc = guide_box.find('div:first');
            var div_backg = div_perc.next();
            var perc = 0;

            guide.fadeIn(500, function() {
                setTimeout(function() { xhr.send(fd); }, 500);
            });
            xhr.upload.onprogress = function(ee) {
                perc = parseInt(ee.loaded / ee.total * 100);
                div_perc.html(perc + '%');
                div_backg.css('width', perc + '%');
            };
            xhr.onload = function() {
                var FILES = xhr.response;
                if(FILES == '') {
                    xhr.onerror();
                    return false;
                }
                div_perc.html('100%');
                form.append('<input type="hidden" name="' + prms.files_varname + '">').find('input:last').val(FILES);
                setTimeout(function() {
                    files_sent = true;
                    form.submit();
                }, prms.submit_delay);
            };
            xhr.onerror = function() {
                guide.fadeOut(500, function() {
                    guide.removeClass('ddup-progressbar');
                    errors.push(lang.err_send_files);
                    posthandle_files(0);
                });
            };
            return false;
        });
        
        return this;
    };
})(jQuery);