(function($) {
    jQuery.fn.gmloc = function(prms) {
        
        prms = $.extend({
            is_changeable: false, //whether user can change map settings
            latitude: 0, //initial latitude
            longitude: 0, //initial longitude
            zoom: 10, //initial zoom level
            pov: {}, //point of street view
            geoname: '', //name of place in marker
            geoname_full: '', //full name of place in serachbox
            show_duration: 2000, //duration of map appearance after its fully loading
            
            //language variables:
            lang_search_box_placeholder: 'Search a Place'
        }, prms);
        
        var obj,
            form,
            gm,
            searchbox,
            marker;
        
        var gmloc = function() {
            //build html:
            this.innerHTML = '<div class="gmloc-gm"></div>';
            obj = this.getElementsByClassName('gmloc-gm')[0];
            form = $(obj).closest('form').get(0);
            
            //get initial coordinates and start to build map:
            var lat = parseFloat(prms.latitude);
            var lng = parseFloat(prms.longitude);
            if(lat == 0 && lng == 0 && navigator.geolocation) {
                navigator.geolocation.getCurrentPosition(function(position) {
                    lat = position.coords.latitude;
                    lng = position.coords.longitude;  
                });
            }
            build_gm(lat, lng);
        }
        
        var build_gm = function(lat, lng) {
            //set coordinates:
            var gm_coords = new google.maps.LatLng(lat, lng);
            
            //create map:
            var gm_prms = {
                zoom: prms.zoom,
                center: gm_coords,
            };
            gm = new google.maps.Map(obj, gm_prms);
            
            //add marker:
            marker = new google.maps.Marker({
                position: gm_coords,
                map: gm,
                title: prms.geoname,
                draggable: prms.is_changeable
            });
            
            search_box();
            
            //show map after its fully loading:
            google.maps.event.addListenerOnce(gm, 'idle', function() {
                $('.gmloc-searchbox').show();
                $(obj).fadeTo(prms.show_duration, 1);
            });
            
            google.maps.event.addListener(gm, 'zoom_changed', function() {
                if(prms.is_changeable) fill_inputs({zoom: gm.getZoom()});
            });
            
            google.maps.event.addListener(marker, 'dragend', function() {
                fill_inputs({geoname: '', position: marker.getPosition()});
            });
            
            street_view();
        };
        
        var search_box = function() {
            $(obj).after('<input class="gmloc-searchbox" type="text" placeholder="' + prms.lang_search_box_placeholder + '" value="' + prms.geoname_full + '">');
            searchbox = document.getElementsByClassName('gmloc-searchbox')[0];
            gm.controls[google.maps.ControlPosition.TOP_LEFT].push(searchbox);
            var sb = new google.maps.places.SearchBox(searchbox);
            
            google.maps.event.addListener(sb, 'places_changed', function() {
                var places = sb.getPlaces();
                if (places.length == 0) return;
                
                //get place data:
                var place = places[0];
                var place_geoloc = place.geometry.location;
                var geoname = place.name;
                var geoname_full = searchbox.value;
                
                //show place:
                var geocoder = new google.maps.Geocoder();
                geocoder.geocode({'address': geoname_full}, function(results, status) {
                    if(status == google.maps.GeocoderStatus.OK) {
                        gm.setCenter(results[0].geometry.location);
                        gm.fitBounds(results[0].geometry.viewport);
                    }
                });
               
                //change marker parameters:
                if(prms.is_changeable) {
                    marker.setOptions({
                        title: geoname,
                        position: place_geoloc
                    });
                    fill_inputs({geoname: geoname, geoname_full: geoname_full, position: marker.getPosition(), zoom: gm.getZoom()});
                }
            });
        };
        
        var street_view = function() {
            //tune street view:
            var sv = gm.getStreetView();
            var sv_opts = {
                linksControl: false,
                enableCloseButton: true
            };
            sv.setOptions(sv_opts);
            
            //open street view if it is set:
            if(!$.isEmptyObject(prms.pov)) {
                var position = new google.maps.LatLng(prms.latitude, prms.longitude);
                sv_opts.position = position;
                sv_opts.pov = prms.pov;
                sv = new google.maps.StreetViewPanorama(obj, sv_opts);
                gm.setStreetView(sv);
            }
            
            //add street view events for user which can change map settings
            if(prms.is_changeable) {
                google.maps.event.addListener(sv, 'visible_changed', function() {
                    if(sv.getVisible()) {
                        fill_inputs({position: sv.getPosition(), pov: sv.getPov()});
                    }
                });

                google.maps.event.addListener(sv, 'pov_changed', function() {
                    fill_inputs({position: sv.getPosition(), pov: sv.getPov()});
                });

                google.maps.event.addListener(sv, 'position_changed', function() {
                    marker.setOptions({
                        position: sv.getPosition()
                    });
                    fill_inputs({geoname: '', position: sv.getPosition(), pov: sv.getPov()});
                });

                google.maps.event.addListener(sv, 'closeclick', function() {
                    fill_inputs({pov: ''});
                });   
            }
        };
        
        //change values of form fields
        var fill_inputs = function(data) {
            if(!prms.is_changeable) return;
            
            for(var prop in data) {
                var val = data[prop];
                switch(prop) {
                    case 'geoname':
                        form.jform_geoname.value = val;
                    break;
                    case 'geoname_full':
                        form.jform_geoname_full.value = val;
                    break;
                    case 'position':
                        form.jform_geolatitude.value = val.lat();
                        form.jform_geolongitude.value = val.lng();
                    break;
                    case 'zoom':
                        form.jform_geozoom.value = val;
                    break;
                    case 'pov':
                        form.jform_geopov.value = val ? JSON.stringify(val) : '';
                    break;
                }
            }
        };
        
        return this.each(gmloc); 
    };
})(jQuery);