<?php
    /*
    * @version      5.3.1 29.03.2016
    * @author       MAXXmarketing GmbH
    * @package      JomGallery
    * @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
    * @license      GNU/GPL
    */
    defined('_JEXEC') or die('Restricted access');

    include_once __DIR__ . '/item_model.php';
    
    class JomGalleryModelGallery extends JomGalleryModelItem_Model {
        
        public function getForm($data = array(), $loadData = true) {
            return $this->loadForm('com_jomgallery.gallery', 'gallery', array('control' => 'jform', 'load_data' => $loadData));
        }
        
        public function loadFormData() {
            $id = JFactory::getApplication()->input->getInt('id');
            if(!$id) return;
            
            $item = parent::getItem($id);
            if(!$item->id) return;
            
            $item->ordering       = $item->id; //for the ordering select
            $item->state          = $item->published;
            $item->cover_file     = $item->cover_filename;
            $item->cover_filename = JGFile::stripExt($item->cover_filename);
            return $item;
        }
        
        public function getTable($type = 'Galleries', $prefix = 'Table', $config = array()) {
            return JTable::getInstance($type, $prefix, $config);
        }
        
        public function save_gallery() {
            $app            = JFactory::getApplication();
            $jinp           = $app->input;
            $id             = $jinp->getInt('id');
            $new            = $id ? false : true;
            $table          = $this->getTable('galleries');
            $table_pictures = $this->getTable('pictures');
            $langs          = JomGalleryHelper::get_langs();
            $mlg_fieldnames = $this->get_mlg_fieldnames();

            $data = new stdClass();
            
            foreach($mlg_fieldnames as $mlg_fieldname) {
                foreach($langs as $lang) {
                    $lang_sfx = '_' . $lang->sef;
                    switch($mlg_fieldname) {
                        case 'alias':
                            $alias = $jinp->get('alias' . $lang_sfx, '', 'raw');
                            $alias = JApplication::stringURLSafe($alias);
                            if( !$alias || is_numeric($alias) ) $alias = JApplication::stringURLSafe($data->{'name' . $lang_sfx});
                            if( !$alias || is_numeric($alias) ) {
                                $alias = date("Y-m-d-H-i-s");
                            }
                            else {
                                $alias_initial = $alias;
                                for($a = 1; $a; $a++) {
                                    if( !$table->tback_check_alias_uniq($id, $alias) ) {
                                        $alias = $alias_initial . '-' . $a;
                                    }
                                    else break;   
                                }
                            }
                            $data->{$mlg_fieldname . $lang_sfx} = $alias;
                        break;
                        case 'description':
                            $description = $jinp->get($mlg_fieldname . $lang_sfx, '', 'raw');
                            $data->{$mlg_fieldname . $lang_sfx} = JComponentHelper::filterText($description);
                        break;
                        default: $data->{$mlg_fieldname . $lang_sfx} = $jinp->getString($mlg_fieldname . $lang_sfx);
                    }
                }
            }
            
            $data->parent_id = $jinp->getInt('parent_id', 0);
            if($id) {
                $ordering = $jinp->getInt('ordering', 0);
                switch($ordering) {
                    case $id: break;
                    case '-1': $data->ordering = 0; break;
                    case '-2': $data->ordering = $table->getNextOrder(); break;
                    default:
                        $ordering_old = $table->tback_get_item_ordering($id);
                        $ordering_new = $table->tback_get_item_ordering($ordering);
                        $data->ordering = ($ordering_new > $ordering_old) ? $ordering_new + 1 : $ordering_new;
                }
            }
            else {
                $data->ordering   = $table->getNextOrder();
            }
            $data->access         = $jinp->getInt('access', 1);
            $data->published      = $jinp->getInt('state', 0);
            $data->modified       = date("Y-m-d H:i:s");
            $data->modified_by    = JFactory::getUser()->id;
            if($new) {
                $data->created    = $data->modified;
                $data->created_by = $data->modified_by;
            }
            if( $created = $jinp->getString('created') ) $data->created = $created;
            if( $created_by = $jinp->getInt('created_by') ) $data->created_by = $created_by;
            
            $success = true;
            if( !$new ) $table->id = $id;
            $table->bind($data);
            if( $table->store() ) {
                $table->reorder();

                $filedata     = JGFile::checkget_filedata('ddup_cover_file');
                $filename_pre = $table->tback_get_cover_filename($id);
                if( $filedata || $filename_pre ) {
                    //create gallery cover filename:
                    $filext       = '.' . ( $filedata ? $filedata->ext : JGFile::get_filext($filename_pre) );
                    $filename     = $table->id . $filext;

                     if( $filename_new = $jinp->getRaw('cover_filename', '', 'RAW') ) {
                         $filename_new = JGFile::makeSafe($filename_new);
                         if( !$filename_new || (is_numeric($filename_new) && $filename_new != $table->id) ) {
                             $app->enqueueMessage(JText::_('COM_JOMGALLERY_ERROR_INCORRECT_FILENAME'), 'warning');
                         }
                         elseif( !$table->tback_check_cover_filename_uniq($table->id, $filename_new . $filext) ) {
                             $app->enqueueMessage(JText::sprintf ('COM_JOMGALLERY_ERROR_FILENAME_EXISTS', $filename_new), 'warning');
                         }
                         else $filename = $filename_new . $filext;
                     }

                     //save or rename gallery cover file:
                     $dirs = JGConfig::_('dirpaths_galls');
                     if( $filedata ) {
                         //save if new file is chosen:
                         if($filename_pre) {
                             foreach($dirs as $dir) {
                                 JGFile::delete($dir . $filename_pre);
                             }
                         }
                         $filedata->id = JGFile::stripExt( $filename );
                         $this->save_image($filedata, JGConfig::_('gallcover_sizes'), $dirs);
                         $table->cover_filename = $filedata->filename;
                         if( !$table->store() ) $success = false;
                     }
                     elseif( $filename_pre && ( $filename != $filename_pre ) ) {
                         //rename if filename is new:
                         $error_rename = false;
                         foreach($dirs as $dir) {
                             if( !JGFile::move($dir . $filename_pre, $dir . $filename) ) $error_rename = true;
                         }
                         $table->cover_filename = $filename;
                         if( !$table->store() ) $error_rename = true;
                         if( $error_rename ) $app->enqueueMessage(JText::_('COM_JOMGALLERY_ERROR_FILE_RENAME'), 'error');
                     }
                }

            }
            else $success = false;

            //reply:
            if($success)
                $app->enqueueMessage(JText::_('COM_JOMGALLERY_GALLERY_MSG_SAVED'), 'message');
            else
                $app->enqueueMessage(JText::_('COM_JOMGALLERY_GALLERY_ERROR_SAVE'), 'error');
            
            return $table->id;
        }
        
        public function delete_gallcover() {
            $id = JFactory::getApplication()->input->getInt('id');
            if( !$id ) return;
            
            $res = true;
            $db = JFactory::getDbo();
            $db->setQuery("UPDATE #__jomgallery_galleries SET cover_filename = '' WHERE id = " . intval($id));
            if( !$db->execute() ) $res = false;
            
            $filename = $this->getTable('galleries')->tback_get_cover_filename($id);
            if( !$filename ) return;
            
            $dirs = JGConfig::_('dirpaths_galls');
            foreach($dirs as $dir) {
                if( !JFile::delete( $dir . $filename ) ) $res = false;
            }
            
            return $res;
        }
        
    }