<?php
    /*
    * @version      5.3.0 24.02.2016
    * @author       MAXXmarketing GmbH
    * @package      JomGallery
    * @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
    * @license      GNU/GPL
    */
    defined('_JEXEC') or die('Restricted access');

    class JomGalleryModelItem_Model extends JModelAdmin {

        public function getForm($data = array(), $loadData = true) {
            return parent::getForm($data, $loadData);
        }
        
        public function check_required_fields() {
            $return = true;
            $app    = JFactory::getApplication();
            $jinp   = $app->input;
            $form   = $this->getForm();
            $xml    = $form->getXml();
            $fields = $xml->xpath('//field');
            $langs  = JomGalleryHelper::get_langs();

            foreach($fields as $field) {
                $required = (string) $field['required'];
                if( $required != 'true' && $required != 'required' ) continue;
                
                $type  = (string) $field['type'];
                $name  = (string) $field['name'];
                $label = (string) $field['label'];
                
                if( $type == 'ddup' ) {
                    if( $jinp->getBool('id') ) continue;
                    
                    $name = 'ddup_' . $name;
                    if( !JGFile::is_uploaded($name) ) {
                        $msg = JText::sprintf('JLIB_FORM_VALIDATE_FIELD_REQUIRED', JText::_($label));
                        $app->enqueueMessage($msg, 'error');
                        $return = false;
                        continue;
                    }
                }
                
                if( strpos($type, 'MLG') === 0 ) {
                    foreach($langs as $lang) {
                        if( isset($_POST['jform'][ $name . '_' . $lang->sef ]) ) continue;
                        
                        $msg = JText::_( $label ) . ' (' . $lang->title_native . ')';
                        $msg = JText::sprintf('JLIB_FORM_VALIDATE_FIELD_REQUIRED', $msg);
                        $app->enqueueMessage($msg, 'error');
                        $return = false;
                    }
                    continue;
                }
                
                if( isset($_POST['jform'][ $name ]) ) continue;
                
                $msg = JText::sprintf('JLIB_FORM_VALIDATE_FIELD_REQUIRED', JText::_($label));
                $app->enqueueMessage($msg, 'error');
                $return = false;
            }
            
            return $return;
        }
        
        public function get_mlg_fieldnames() {
            $form           = $this->getForm();
            $xml            = $form->getXml();
            $fields         = $xml->xpath('//field');
            $mlg_fieldnames = array();
            
            foreach($fields as $field) {
                if( strpos((string) $field['type'], 'MLG') === 0 ) {
                    $mlg_fieldnames[] = (string) $field['name'];
                }
            }
            
            return $mlg_fieldnames;
        }

        public function save_image($fdata, $sizes, $dirs) {
            $app = JFactory::getApplication();
            //create fullsize picture:
            $jimage = new JImage($fdata->tmp_name);
            $width = $jimage->getWidth();
            $height = $jimage->getHeight();
            if($width > $sizes->full_width || $height > $sizes->full_height) {
                $scale_method = ($width > $height) ? JImage::SCALE_INSIDE : JImage::SCALE_OUTSIDE;
                $jimage->resize($sizes->full_width, $sizes->full_height, false, $scale_method);
            }
            //save fullsize picture:
            $fdata->filename = $fdata->id . '.' . $fdata->ext;
            $path_full = $dirs->fulls . $fdata->filename;
            $jimage->toFile($path_full, $fdata->imgtype);
            if( !file_exists($path_full) ) $app->enqueueMessage(JText::sprintf('COM_JOMGALLERY_ERROR_FILE_SAVE', $fdata->name), 'error');
            //impose watermarksave on fullsize picture:
            $this->impose_watermark($path_full);
            
            //create middle picture and thumb:
            $jimage = new JImage($fdata->tmp_name);
            $jimages_sizes = array($sizes->middle_width . 'x' . $sizes->middle_height, $sizes->thumb_width . 'x' . $sizes->thumb_height);
            $mid_and_thumb = $jimage->generateThumbs($jimages_sizes, JImage::CROP_RESIZE);
            
            //save middle picture:
            $path_middle = $dirs->middles . $fdata->filename;
            $jimage_middle = $mid_and_thumb[0];
            $jimage_middle->toFile($path_middle, $fdata->imgtype);
            if( !file_exists($path_middle) ) $app->enqueueMessage(JText::sprintf('COM_JOMGALLERY_ERROR_FILE_SAVE_MIDDLE', $fdata->name), 'error');
            //impose watermark on middle picture:
            $scale_factor = ($sizes->middle_width + $sizes->middle_height) / ($sizes->full_width + $sizes->full_height);
            $this->impose_watermark($path_middle, $scale_factor);
            
            //save thumb:
            $path_thumb = $dirs->thumbs . $fdata->filename;
            $jimage_thumb = $mid_and_thumb[1];
            $jimage_thumb->toFile($path_thumb, $fdata->imgtype);
            if( !file_exists($path_thumb) ) $app->enqueueMessage(JText::sprintf('COM_JOMGALLERY_ERROR_FILE_SAVE_THUMB', $fdata->name), 'error');
        }
        
        public function impose_watermark($path, $scaling_factor = 1) {
            $app = JFactory::getApplication();
            $controller = $app->input->getCmd('controller');
            if( !JGConfig::_('watermark_on_' . $controller) ) return;
            
            $error = JText::_('COM_JOMGALLERY_CONFIG_WATERMARK_ERROR_IMPOSE');
            $ext = JGFile::get_filext($path);
            $dst_imagecreator = 'imagecreatefrom' . $ext;
            $wm_imagecreator = 'imagecreatefrom' . JGFile::get_filext(JGConfig::_('watermark_filename'));
            $wm_path = JGConfig::_('dirpath') . JGConfig::_('watermark_filename');
            
            //create file's and watermark's gd resources and check them:
            if( !is_resource( $dst = $dst_imagecreator($path) ) || !is_resource( $wm = $wm_imagecreator($wm_path) ) ) {
                $app->enqueueMessage($error, 'error');
                return;
            }
            
            //define images' values:
            $wm_w_orig = imagesx($wm);
            $wm_h_orig = imagesy($wm);
            $wm_w = $wm_w_orig * $scaling_factor;
            $wm_h = $wm_h_orig * $scaling_factor;
            $dst_w = imagesx($dst);
            $dst_h = imagesy($dst);
            $dst_x = JGConfig::_('watermark_indent') * $scaling_factor;
            $dst_y = $dst_x;
            
            //calculate watermark location:
            switch( JGConfig::_('watermark_location') ) {
                case 'left_top': break;
                case 'right_top':
                    $dst_x = $dst_w - $wm_w - $dst_x;
                break;
                case 'center':
                    $dst_x = ($dst_w / 2) - ($wm_w / 2);
                    $dst_y = ($dst_h / 2) - ($wm_h / 2);
                break;
                case 'left_bottom':
                    $dst_y = $dst_h - $wm_h - $dst_y;
                break;
                case 'right_bottom':
                    $dst_x = $dst_w - $wm_w - $dst_x;
                    $dst_y = $dst_h - $wm_h - $dst_y;
                break;              
            }
            
            //impose watermark and save file:
            imagecopyresampled($dst, $wm, $dst_x, $dst_y, 0, 0, $wm_w, $wm_h, $wm_w_orig, $wm_h_orig);
            switch($ext) {
                case 'jpeg': imagejpeg($dst, $path); break;
                case 'gif': imagegif($dst, $path); break;
                case 'png':
                    imagesavealpha($dst, true);
                    imagepng($dst, $path);
            }
            
            //destroy gd resources:
            imagedestroy($wm);
            imagedestroy($dst);
            
            if(!file_exists($path)) $app->enqueueMessage($error, 'error');
        }
        
        public function rotate() {
            $jinp = JFactory::getApplication()->input;
            $controller = $jinp->getCmd('controller');
            $task = $jinp->getCmd('task');
            $id = $jinp->getInt('id');
            $dirs = JGConfig::_('dirpaths_' . ( ($controller == 'gallery') ? 'galls' : 'pics' ) );
            $filename = ($controller == 'gallery') ? $this->getTable('galleries')->tback_get_cover_filename($id) : $this->getTable('pictures')->tback_get_filename($id);
            if( !$id || !$filename || ($task != 'rotate_clockwise' && $task != 'rotate_counterclockwise') ) return;

            $ext = JGFile::get_filext($dirs->thumbs . $filename, true);
            $type = constant('IMAGETYPE_' . strtoupper($ext));
            $angle = ($task == 'rotate_clockwise') ? 270 : 90;
            foreach($dirs as $dir) {
                $filepath = $dir .  $filename;
                if( !file_exists($filepath) ) continue;
                
                $jimage = new JImage($filepath);
                $rotated_image = $jimage->rotate($angle);
                $rotated_image->toFile($filepath, $type);
            }
        }
        
    }
?>