<?php
    /*
    * @version      5.3.1 29.03.2016
    * @author       MAXXmarketing GmbH
    * @package      JomGallery
    * @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
    * @license      GNU/GPL
    */
    defined('_JEXEC') or die('Restricted access');

    include_once __DIR__ . '/item_model.php';
    
    class JomGalleryModelPicture extends JomGalleryModelItem_Model {

        public function getForm($data = array(), $loadData = true) {
            return $this->loadForm('com_jomgallery.picture', 'picture', array('control' => 'jform', 'load_data' => $loadData));
        }
        
        public function loadFormData() {
            $id = JFactory::getApplication()->input->getInt('id');
            if(!$id) return;
            
            $item = parent::getItem($id);
            if(!$item->id) return;
            
            $item->ordering = $item->id; //for the ordering select
            $item->state    = $item->published;
            $item->file     = $item->filename;
            $item->filename = JGFile::stripExt($item->file);
            return $item;
        }
        
        public function getTable($type = 'Pictures', $prefix = 'Table', $config = array()) {
            return JTable::getInstance($type, $prefix, $config);
        }
        
        public function save_picture() {
            $app            = JFactory::getApplication();
            $jinp           = $app->input;
            $id             = $jinp->getInt('id');
            $new            = $id ? false : true;
            $table          = $this->getTable('pictures');
            $langs          = JomGalleryHelper::get_langs();
            $mlg_fieldnames = $this->get_mlg_fieldnames();
            $filedata       = JGFile::checkget_filedata('ddup_file');
            
            if( $new && !$filedata) {
                $app->enqueueMessage(JText::sprintf('COM_JOMGALLERY_ERROR_FILE_', 'unknown file'), 'error');
                return;
            }
            
            //prepare data:
            $data = new stdClass();
            
            foreach($mlg_fieldnames as $mlg_fieldname) {
                foreach($langs as $lang) {
                    $lang_sfx = '_' . $lang->sef;
                    switch($mlg_fieldname) {
                        case 'alias':
                            $alias = $jinp->get('alias' . $lang_sfx, '', 'raw');
                            $alias = JApplication::stringURLSafe($alias);
                            if( !$alias || is_numeric($alias) ) $alias = JApplication::stringURLSafe($data->{'name' . $lang_sfx});
                            if( !$alias || is_numeric($alias) ) {
                                $alias = date("Y-m-d-H-i-s");
                            }
                            else {
                                $alias_initial = $alias;
                                for($a = 1; $a; $a++) {
                                    if( !$table->tback_check_alias_uniq($id, $alias) ) {
                                        $alias = $alias_initial . '-' . $a;
                                    }
                                    else break;   
                                }
                            }
                            $data->{$mlg_fieldname . $lang_sfx} = $alias;
                        break;
                        case 'description':
                            $description = $jinp->get($mlg_fieldname . $lang_sfx, '', 'raw');
                            $data->{$mlg_fieldname . $lang_sfx} = JComponentHelper::filterText($description);
                        break;
                        default: $data->{$mlg_fieldname . $lang_sfx} = $jinp->getString($mlg_fieldname . $lang_sfx);
                    }
                }
            }
            
            if( !$new ) {
                $ordering = $jinp->getInt('ordering', 0);
                switch($ordering) {
                    case $id: break;
                    case '-1': $data->ordering = 0; break;
                    case '-2': $data->ordering = $table->getNextOrder(); break;
                    default:
                        $data->ordering = $table->tback_get_item_ordering($ordering);
                        if($data->ordering > $table->tback_get_item_ordering($id)) $data->ordering++;
                }
            }
            else $data->ordering  = $table->getNextOrder();
            
            $data->parent_id      = $jinp->getInt('parent_id');
            $data->published      = $jinp->getInt('state', 0);
            $data->access         = $jinp->getInt('access', 1);
            $data->geolocation    = $jinp->getInt('geolocation');
            $data->geoname        = $jinp->getString('geoname');
            $data->geoname_full   = $jinp->getString('geoname_full');
            $data->geolatitude    = $jinp->getFloat('geolatitude');
            $data->geolongitude   = $jinp->getFloat('geolongitude');
            $data->geozoom        = $jinp->getInt('geozoom');
            $data->geopov         = $jinp->getString('geopov');
            $data->modified       = date("Y-m-d H:i:s");
            $data->modified_by    = JFactory::getUser()->id;
            if($new) {
                $data->created    = $data->modified;
                $data->created_by = $data->modified_by;
            }
            if( $created = $jinp->getString('created') ) $data->created = $created;
            if( $created_by = $jinp->getInt('created_by') ) $data->created_by = $created_by;
            
            //save data to database:
            $success = true;
            if( !$new ) $table->id = $id;
            $table->bind($data);
            if( $table->store() ) {
                $table->reorder();

                //create filename:
                $filename_pre = $table->tback_get_filename($id);
                $filext       = '.' . ( $filedata ? $filedata->ext : JGFile::get_filext($filename_pre) );
                $filename     = $table->id . $filext;

                if( $filename_new = $jinp->get('filename', '', 'RAW') ) {
                    $filename_new = JGFile::makeSafe($filename_new);
                    if( !$filename_new || (is_numeric($filename_new) && $filename_new != $table->id) ) {
                        $app->enqueueMessage(JText::_('COM_JOMGALLERY_ERROR_INCORRECT_FILENAME'), 'warning');
                    }
                    elseif( !$table->tback_check_filename_uniq($table->id, $filename_new . $filext) ) {
                        $app->enqueueMessage(JText::sprintf ('COM_JOMGALLERY_ERROR_FILENAME_EXISTS', $filename_new), 'warning');
                    }
                    else $filename = $filename_new . $filext;
                }

                //save or rename picture file:
                $dirs = JGConfig::_('dirpaths_pics');
                if( $filedata ) {
                    //save if new file is chosen:
                    if($filename_pre) {
                        foreach($dirs as $dir) {
                            JGFile::delete($dir . $filename_pre);
                        }
                    }
                    $filedata->id = JGFile::stripExt( $filename );
                    $this->save_image($filedata, JGConfig::_('pic_sizes'), $dirs);
                    $table->filename = $filedata->filename;
                    if( !$table->store() ) $success = false;
                }
                elseif( $filename != $filename_pre ) {
                    //rename if filename is new:
                    $error_rename = false;
                    foreach($dirs as $dir) {
                        if( !JGFile::move($dir . $filename_pre, $dir . $filename) ) $error_rename = true;
                    }
                    $table->filename = $filename;
                    if( !$table->store() ) $error_rename = true;
                    if( $error_rename ) $app->enqueueMessage(JText::_('COM_JOMGALLERY_ERROR_FILE_RENAME'), 'error');
                }
            }
            else $success = false;
            
            //reply:
            if($success)
                $app->enqueueMessage(JText::_('COM_JOMGALLERY_PICTURE_MSG_SAVED'));
            else
                $app->enqueueMessage(JText::_('COM_JOMGALLERY_PICTURE_ERROR_SAVE'), 'error');
            
            return $table->id;
        }
        
        public function save_pictures() {
            $app               = JFactory::getApplication();
            $jinp              = $app->input;
            $table             = $this->getTable('pictures');
            $table_galleries   = $this->getTable('galleries');
            $langs             = JomGalleryHelper::get_langs();
            $mlg_fieldnames    = array('name', 'alias');
            $dirs              = JGConfig::_('dirpaths_pics');
            
            //prepare data:
            $data              = new stdClass();       
            $date              = date("Y-m-d H:i:s");
            $user_id           = JFactory::getUser()->id;
            $data->parent_id   = $jinp->getInt('parent_id');
            $data->ordering    = $table->getNextOrder();
            $data->created     = $date;
            $data->created_by  = $user_id;
            $data->modified    = $date;
            $data->modified_by = $user_id;
            $data->published   = $jinp->getInt('state', 0);
            $data->access      = $jinp->getInt('access', 1);
            
            //check files and save valid:
            $saved = 0;
            $files_sum = count($_FILES['ddup_pictures']['name'][JGConfig::_('files_arrname')]);
            for($i = 0; $i < $files_sum; $i++) {
                //extract file data and check it:
                $filedata = JGFile::checkget_filedata('ddup_pictures', $i);
                if( !$filedata ) continue;
                
                //define individual data (name and unique alias):
                $name = substr($filedata->name, 0, strrpos($filedata->name, '.'));
                $alias = JApplication::stringURLSafe($name);
                if( !$alias || is_numeric($alias) ) {
                    $alias = date("Y-m-d-H-i-s") . '-' . $i;
                }
                else {
                    $alias_initial = $alias;
                    for($a = 1; $a; $a++) {
                        if(
                            !$table->tback_check_alias_uniq(null, $alias) ||
                            !$table_galleries->tback_check_alias_uniq(null, $alias)
                        ) {
                            $alias = $alias_initial . '-' . $a;
                        }
                        else break;   
                    }  
                }
                foreach($mlg_fieldnames as $mlg_fieldname) {
                    foreach($langs as $lang) {
                        $data->{$mlg_fieldname . '_' . $lang->sef} = ${$mlg_fieldname};
                    }
                }

                //save data to database:
                $table->id = null;
                $table->reset();
                $table->bind($data);
                if( !$table->store() ) {
                    $app->enqueueMessage(JText::sprintf('COM_JOMGALLERY_ERROR_FILE_SAVE', $data->name), 'error');
                    continue;
                }

                //save file:
                $filedata->id = $table->id;
                $this->save_image($filedata, JGConfig::_('pic_sizes'), $dirs);

                //save filename to database:
                $table->reset();
                $table->filename = $filedata->filename;
                $table->store();
                
                $saved++;
            }
            
            //reorder table and reply:
            if($saved) {
                $table->reorder();
                $app->enqueueMessage( ($saved > 1) ? JText::plural('COM_JOMGALLERY_PICTURES_MSG_N_ITEMS_SAVED', $saved) : JText::_('COM_JOMGALLERY_PICTURE_MSG_SAVED') );
                return true;
            }
        }
        
    }
?>