<?php
    /*
    * @version      5.2.0 22.02.2016
    * @author       MAXXmarketing GmbH
    * @package      JomGallery
    * @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
    * @license      GNU/GPL
    */
    defined('_JEXEC') or die('Restricted access');
    
    include_once __DIR__ . '/jomgallery.php';
    
    class TableGalleries extends TableJomgallery {
        
        function __construct(&$db) {
            parent::__construct('#__jomgallery_galleries', 'id', $db);
        }

        public function tback_check_alias_uniq($id, $alias) {
            $langs = JomGalleryHelper::get_langs();
            foreach($langs as $lang) {
                $query = $this->_db->getQuery(true)
                        ->select('id')
                        ->from($this->_tbl)
                        ->where('id != ' . intval($id))
                        ->where($this->_db->qn('alias_' . $lang->sef) . ' LIKE ' . $this->_db->q($alias));
                $this->_db->setQuery($query);
                if( $this->_db->loadColumn() ) return;
            }
            return $alias;
        }
        
        public function tback_get_cover_filename($id) {
            $query = $this->_db->getQuery(true)
                    ->select('cover_filename')
                    ->from($this->_tbl)
                    ->where('id = ' . intval($id));
            $this->_db->setQuery($query);
            return $this->_db->loadResult();
        }
        
        public function tback_check_cover_filename_uniq($id, $cover_filename) {
            if(!$cover_filename) return;
            $query = $this->_db->getQuery(true)
                    ->select('id')
                    ->from($this->_tbl)
                    ->where('id != ' . intval($id))
                    ->where('cover_filename LIKE ' . $this->_db->quote($cover_filename));
            $this->_db->setQuery($query);
            return $this->_db->loadColumn() ? null : $cover_filename;
        }
        
        public function tback_check_nesting($id) {
            if(!$id = intval($id)) return;
            
            $query = $this->_db->getQuery(true)
                    ->select('pg.id')
                    ->from($this->_tbl . ' AS g')
                    ->innerJoin($this->_tbl . ' AS pg ON pg.id = g.parent_id')
                    ->where('g.id = ' . $id)
                    ->where('g.published = 1');
            $this->_db->setQuery($query);
            return $this->_db->loadResult();
        }

        public function tback_get_galleries($state_props) {
            $jinp = JFactory::getApplication()->input;
            $to_contr = new stdClass();
            
            $query = $this->_db->getQuery(true)
                    ->select('
                        g.id,
                        g.name_' . $this->langsef . ' AS name,
                        g.cover_filename AS cover,
                        g.ordering,
                        g.created,
                        g.created_by,
                        g.published,
                        g.hits
                    ')
                    ->from($this->_tbl . ' AS g')
                    ->select('g.parent_id, gp.name_' . $this->langsef . ' AS parent_name')
                    ->leftJoin($this->_tbl . ' AS gp ON gp.id = g.parent_id')
                    ->select('vls.title AS access')
                    ->leftJoin('#__viewlevels AS vls ON vls.id = g.access')
                    ->select('u.name AS created_by_name')
                    ->leftJoin('#__users AS u ON u.id = g.created_by');
            
            $active_filters = array();
            foreach($state_props as $k => $v) {
                if(strpos($k, 'filter.') !== 0) continue;
                $k = substr($k, 7);
                $jinp->set($k, $v);
                switch($k) {
                    case 'search':
                        if(!$v = $jinp->getString($k)) break;
                        $v = $this->_db->quote('%' . str_replace(' ', '%', $this->_db->escape(trim($v), true)) . '%', false);
                        $query->where('(g.name_' . $this->langsef . ' LIKE ' . $v . ' OR g.description_short_' . $this->langsef . ' LIKE ' . $v . ' OR g.description_' . $this->langsef . ' LIKE ' . $v . ')');
                    break;
                    case 'parent_id':
                        if(!is_numeric($v) || ((!$v = $jinp->getCmd($k)) && $v !== '0')) break;
                        $query->where('g.parent_id = ' . $v);
                    break;
                    case 'published':
                        if(!is_numeric($v)) break;
                        $v = $jinp->getInt($k);
                        $query->where('g.published = ' . $v);
                    break;
                    case 'access':
                        if(!is_numeric($v) || !$v = $jinp->getInt($k)) break;
                        $query->where('g.access = ' . $v);
                    break;
                    case 'author':
                        if(!is_numeric($v) || !$v = $jinp->getInt($k)) break;
                        $query->where('g.created_by = ' . $v);
                    break;
                }
                if($k != 'search' && ($v || $v === '0' || $v === 0)) $active_filters[$k] = $v;
            }

            $to_contr->active_filters = $active_filters;
            $to_contr->filter_order = isset($state_props['list.ordering']) ? $state_props['list.ordering'] : 'g.ordering';
            $to_contr->filter_order_dir = isset($state_props['list.direction']) ? $state_props['list.direction'] : 'ASC';
            $to_contr->fullordering = $to_contr->filter_order . ' ' . $to_contr->filter_order_dir;
            $query->order($to_contr->fullordering);

            $res = new stdClass();
            $res->query = $query;
            $res->to_contr = $to_contr;
            return $res;
        }
        
        public function tback_get_progeny($id) {
            if(!$id = intval($id)) return;
            
            $progeny = array($id);
            $check_ids = array($id);
            do {
                $cur_id = array_shift($check_ids);
                $query = $this->_db->getQuery(true)
                        ->select('id')
                        ->from($this->_tbl)
                        ->where('parent_id = ' . $cur_id);
                $this->_db->setQuery($query);
                $generation = $this->_db->loadColumn();
                JArrayHelper::toInteger($generation);
                $progeny = array_merge($progeny, $generation);
                $check_ids = array_merge($check_ids, $generation);
            }
            while($check_ids);
            
            return $progeny;            
        }
        
        public function tback_get_item_ordering($id) {
            $query = $this->_db->getQuery(true)
                    ->select('ordering')
                    ->from($this->_tbl)
                    ->where('id = ' . intval($id));
            $this->_db->setQuery($query);
            return $this->_db->loadResult();
        }
        
        public function tfront_get_galleries($id = null) {
            $query = $this->_db->getQuery(true)
                    ->select('
                        g.id,
                        g.name_' . $this->langsef . ' AS name,
                        g.alias_' . $this->langsef . ' AS alias,
                        g.description_short_' . $this->langsef . ' AS description_short,
                        g.ordering,
                        g.cover_filename,
                        g.cover_alt_' . $this->langsef . ' AS cover_alt,
                        g.created,
                        g.created_by,
                        g.modified,
                        g.modified_by,
                        g.hits,
                        g.published,
                        g.metatitle_' . $this->langsef . ' AS metatitle,
                        g.metakey_' . $this->langsef . ' AS metakey,
                        g.metadesc_' . $this->langsef . ' AS metadesc
                    ')
                    ->from($this->_tbl . ' AS g')
                    ->where('g.parent_id = ' . (($id = intval($id)) ? $id : 0))
                    ->where('g.published = 1')
                    ->where('g.' . $this->query_access)
                    ->order('g.ordering ASC');
            $this->_db->setQuery($query);
            return $this->_db->loadObjectList();
        }
        
        public function tfront_get_gallery($id) {
            $query = $this->_db->getQuery(true)
                    ->select('
                        id,
                        parent_id,
                        name_' . $this->langsef . ' AS name,
                        alias_' . $this->langsef . ' AS alias,
                        cover_filename,
                        cover_alt_' . $this->langsef . ' AS cover_alt,
                        hits,
                        description_short_' . $this->langsef . ' AS description_short,
                        description_' . $this->langsef . ' AS description,
                        DATE_FORMAT(created, "%m.%d.%Y") AS created,
                        access,
                        metatitle_' . $this->langsef . ' AS metatitle,
                        metakey_' . $this->langsef . ' AS metakey,
                        metadesc_' . $this->langsef . ' AS metadesc
                    ')
                    ->from($this->_tbl)
                    ->where('id = ' . intval($id))
                    ->where('published = 1');
            $this->_db->setQuery($query);
            $gallery = $this->_db->loadObject();
            
            if( !$this->check_parental_galls($gallery->parent_id) ) return;
            
            return $gallery;
        }
        
    }