<?php
    /*
    * @version      5.2.0 22.02.2016
    * @author       MAXXmarketing GmbH
    * @package      JomGallery
    * @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
    * @license      GNU/GPL
    */
    defined('_JEXEC') or die('Restricted access');
    
    include_once __DIR__ . '/jomgallery.php';
    
    class TablePictures extends TableJomgallery {
        
        function __construct(&$db) {
            parent::__construct('#__jomgallery_pictures', 'id', $db);
        }
        
        public function tback_check_alias_uniq($id, $alias) {
            $langs = JomGalleryHelper::get_langs();
            foreach($langs as $lang) {
                $query = $this->_db->getQuery(true)
                        ->select('id')
                        ->from($this->_tbl)
                        ->where('id != ' . intval($id))
                        ->where($this->_db->qn('alias_' . $lang->sef) . ' LIKE ' . $this->_db->q($alias));
                $this->_db->setQuery($query);
                if( $this->_db->loadColumn() ) return;
            }
            return $alias;
        }
        
        public function tback_get_filename($id) {
            $query = $this->_db->getQuery(true)
                    ->select('filename')
                    ->from($this->_tbl)
                    ->where('id = ' . intval($id));
            $this->_db->setQuery($query);
            return $this->_db->loadResult();
        }
        
        public function tback_check_filename_uniq($id, $filename) {
            if(!$filename) return;
            $query = $this->_db->getQuery(true)
                    ->select('id')
                    ->from($this->_tbl)
                    ->where('id != ' . intval($id))
                    ->where('filename LIKE ' . $this->_db->quote($filename));
            $this->_db->setQuery($query);
            return $this->_db->loadColumn() ? null : $filename;
        }
        
        public function tback_get_pictures($state_props) {
            $jinp = JFactory::getApplication()->input;
            $to_contr = new stdClass();
            
            $query = $this->_db->getQuery(true)
                    ->select('p.id, p.name_' . $this->langsef . ' AS name, p.parent_id, p.filename, p.ordering, p.created, p.created_by, p.published, p.access, p.hits')
                    ->from($this->_tbl . ' AS p')
                    ->select('g.name_' . $this->langsef . ' AS gallery_name')
                    ->leftJoin('#__jomgallery_galleries AS g ON g.id = p.parent_id')
                    ->select('vls.title AS access')
                    ->leftJoin('#__viewlevels AS vls ON vls.id = p.access')
                    ->select('u.name AS created_by_name')
                    ->leftJoin('#__users AS u ON u.id = p.created_by');
            
            $active_filters = array();
            foreach($state_props as $k => $v) {
                if(strpos($k, 'filter.') !== 0) continue;
                $k = substr($k, 7);
                $jinp->set($k, $v);
                switch($k) {
                    case 'search':
                        if(!$v = $jinp->getString($k)) break;
                        $v = $this->_db->quote('%' . str_replace(' ', '%', $this->_db->escape(trim($v), true)) . '%', false);
                        $query->where('(p.name_' . $this->langsef . ' LIKE ' . $v . ' OR p.description_' . $this->langsef . ' LIKE ' . $v . ')');
                    break;
                    case 'gallery':
                        if(!is_numeric($v) || ((!$v = $jinp->getCmd($k)) && $v !== '0')) break;
                        $query->where('p.parent_id = ' . $v);
                    break;
                    case 'published':
                        if(!is_numeric($v)) break;
                        $v = $jinp->getInt($k);
                        $query->where('p.published = ' . $v);
                    break;
                    case 'access':
                        if(!is_numeric($v) || !$v = $jinp->getInt($k)) break;
                        $query->where('p.access = ' . $v);
                    break;
                    case 'author':
                        if(!is_numeric($v) || !$v = $jinp->getInt($k)) break;
                        $query->where('p.created_by = ' . $v);
                    break;
                }
                if($k != 'search' && ($v || $v === '0' || $v === 0)) $active_filters[$k] = $v;
            }

            $to_contr->active_filters = $active_filters;
            $to_contr->filter_order = isset($state_props['list.ordering']) ? $state_props['list.ordering'] : 'p.ordering';
            $to_contr->filter_order_dir = isset($state_props['list.direction']) ? $state_props['list.direction'] : 'ASC';
            $to_contr->fullordering = $to_contr->filter_order . ' ' . $to_contr->filter_order_dir;
            $query->order($to_contr->fullordering);
            
            $res = new stdClass();
            $res->query = $query;
            $res->to_contr = $to_contr;
            return $res;
        }

        public function tback_get_item_ordering($id) {
            $query = $this->_db->getQuery(true)
                    ->select('ordering')
                    ->from($this->_tbl)
                    ->where('id = ' . intval($id));
            $this->_db->setQuery($query);
            return $this->_db->loadResult();
        }
        
        public function tfront_get_pictures($id) {
            $query = $this->_db->getQuery(true)
                    ->select('
                        id,
                        name_' . $this->langsef . ' AS name,
                        filename,
                        alt_' . $this->langsef . ' AS alt
                    ')
                    ->from($this->_tbl)
                    ->where('parent_id = ' . intval($id))
                    ->where('published = 1')
                    ->where($this->query_access)
                    ->order('ordering ASC');
            $this->_db->setQuery($query);
            return $this->_db->loadObjectList();
        }
        
        public function tfront_get_picture($id) {
            $gallery_picsum = '(SELECT COUNT(*) FROM ' . $this->_tbl . ' WHERE parent_id = p.parent_id';
            $query = $this->_db->getQuery(true)
                ->select('
                    p.id,
                    p.parent_id,
                    p.name_' . $this->langsef . ' AS name,
                    p.alias_' . $this->langsef . ' AS alias,
                    p.filename,
                    p.alt_' . $this->langsef . ' AS alt,
                    p.description_' . $this->langsef . ' AS description,
                    p.ordering,
                    DATE_FORMAT(p.created, "%m.%d.%Y") AS created,
                    p.access,
                    p.hits,
                    p.geolocation,
                    p.geoname,
                    p.geoname_full,
                    p.geolatitude,
                    p.geolongitude,
                    p.geozoom,
                    p.geopov,
                    p.metatitle_' . $this->langsef . ' AS metatitle,
                    p.metakey_' . $this->langsef . ' AS metakey,
                    p.metadesc_' . $this->langsef . ' AS metadesc
                ')
                ->select($gallery_picsum . ' AND ordering <= p.ordering GROUP BY parent_id ORDER BY ordering ASC) AS index_number')
                ->select($gallery_picsum . ' GROUP BY parent_id) AS gallery_picsum')
                ->from($this->_tbl . ' AS p')
                ->where('p.id = ' . intval($id))
                ->where('p.published = 1');
            $this->_db->setQuery($query);
            $picture = $this->_db->loadObject();
            
            if( !$picture ) return;
            if( !$this->check_parental_galls($picture->parent_id) ) return;
            
            $picture->prev_picture = $this->_get_nearby_pictire(0, $picture->parent_id, $picture->ordering);
            $picture->next_picture = $this->_get_nearby_pictire(1, $picture->parent_id, $picture->ordering);
            return $picture;
        }
        
        private function _get_nearby_pictire($direction, $parent_id, $ordering) {
            $more_or_less = ($direction) ? '>' : '<';
            $order_dir = ($direction) ? 'ASC' : 'DESC';
            $query = $this->_db->getQuery(true)
                ->select("id")
                ->from($this->_tbl)
                ->where('parent_id = ' . intval($parent_id))
                ->where('ordering ' . $more_or_less . intval($ordering))
                ->where('published = 1')
                ->order('ordering ' . $order_dir);
            $this->_db->setQuery($query, 0, 1);
            return $this->_db->loadObject();
        }
    }